import os
import pathlib
import typing as t

import toml


home_config = pathlib.Path("~/.judorc").expanduser()
project_config = pathlib.Path(".judorc")


def _load_file(path: t.Union[str, os.PathLike]) -> dict:
    if not isinstance(path, pathlib.Path):
        path = pathlib.Path(path)

    if not path.exists():
        return {}

    if not path.is_file():
        return {}

    with path.open() as f:
        return toml.load(f)


def load(*paths):
    config = {}

    # by default loads the judoka config present in the home folder
    # setting the environment variable to JUDOKA_HOME="" disables it
    config.update(_load_file(path=os.getenv("JUDOKA_HOME", home_config)))

    config.update(_load_file(path=project_config))

    for path in paths:
        config.update(_load_file(path))

    return config



from functools import reduce
import operator
import subprocess
import typing as t

from click import (
    argument,
    command,
    Command,
    Context,
    MultiCommand,
)


class Hub(MultiCommand):
    _config: t.Optional[dict]
    _configresolver: t.Optional[t.Callable[[], dict]]

    def __init__(
        self,
        *args,
        config: t.Optional[t.Mapping] = None,
        configresolver: t.Optional[t.Callable[[], t.Mapping]] = None,
        **kwargs,
    ):
        self._config = config
        self._configresolver = configresolver
        super().__init__(*args, **kwargs)

    @property
    def config(self):
        if not self._config and self._configresolver:
            self._config = self._configresolver()

        return self._config

    def list_commands(self, ctx: Context) -> t.List[str]:
        commands = []

        for key, value in self.config.items():
            if type(value) is str:
                # if value is str, consider the key the name of the command
                commands.append(key)
            elif type(value) is dict:
                # if the value is a dict, consider the key a namespace
                # all keys within the dict are the names of the commands
                for name in value.keys():
                    commands.append(f"{key}:{name}")

        commands.sort()

        return commands

    def get_command(self, ctx: Context, cmd_name: str) -> t.Optional[Command]:
        if cmd_name not in self.list_commands(ctx):
            return None

        args = self.get_command_args(cmd_name)
        if type(args) == list:
            args = " ".join(args)

        @command(name=cmd_name)
        @argument("shell_args", nargs=-1)
        def _command(shell_args):
            process = subprocess.Popen(
                args + " " + " ".join(shell_args),
                shell=True,
            )
            process.wait()
            process.kill()

        return _command

    def get_command_args(self, cmd_name):
        try:
            args = reduce(
                operator.getitem,
                cmd_name.split(":"),
                self.config,
            )
        except KeyError:
            args = []
        return args



import pathlib
import subprocess
from functools import partial
from importlib import resources
from shutil import copy

import click


def rc_installer(rc_file, completion_file):
    judoka_homedir = pathlib.Path("~/.judo").expanduser()
    judoka_homedir.mkdir(exist_ok=True)

    with resources.path("judoka", "completions") as path:
        completion_target = copy(path / completion_file, judoka_homedir)

    return f"echo 'source {completion_target}' >> {rc_file}"


def path_installer(install_path, completion_file):
    install_path.mkdir(exist_ok=True)

    with resources.path("completions") as path:
        copy(path / completion_file, install_path)


installers = {
    "bash": partial(
        rc_installer,
        rc_file=pathlib.Path("~/.bashrc").expanduser(),
        completion_file="judo.bash",
    ),
    "zsh": partial(
        rc_installer,
        rc_file=pathlib.Path("~/.zshrc").expanduser(),
        completion_file="judo.zsh",
    ),
    "fish": partial(
        path_installer,
        install_path=pathlib.Path("~/.config/fish/completions/").expanduser(),
        completion_file="judo.fish",
    ),
}


@click.command()
@click.argument("shell", type=click.Choice(["bash", "fish", "zsh"]))
@click.option("--apply/--no-apply", default=False)
def install(shell, apply=False):
    installer = installers.get(shell)

    if not installer:
        click.echo(f"No installer found for shell '{shell}'")
        exit(1)

    instruction = installer()
    if instruction:
        if not apply:
            click.echo("\nTo complete installation, execute the following command:\n")
            click.echo(instruction)
            click.echo("\nAlternatively, run with --force to automate installation.\n")
        else:
            subprocess.run(instruction, shell=True)
            click.echo(f"Shell completion was installed for you via: \n{instruction}")
    else:
        click.echo("Shell completion was installed")


if __name__ == "__main__":
    install()



import setuptools
from setuptools import setup
import os

VERSION = "0.3.1"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="judoka",
    description="",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Eelke van den Bos",
    url="https://github.com/eelkevdbos/judoka",
    project_urls={
        "Issues": "https://github.com/eelkevdbos/judoka/issues",
        "CI": "https://github.com/eelkevdbos/judoka/actions",
        "Changelog": "https://github.com/eelkevdbos/judoka/releases",
    },
    license="MIT",
    version=VERSION,
    packages=setuptools.find_packages("src"),
    include_package_data=True,
    package_dir={"": "src"},
    package_data={"judoka": ["completions/*"]},
    entry_points={
        "console_scripts": [
            "judo=judoka.cli:hub",
            "judo-completions=judoka.completion:install",
        ]
    },
    install_requires=["toml", "click"],
    extras_require={"test": ["pytest"]},
    python_requires=">=3.6",
)
